@@ -30,17 +30,15 @@
 
 #include "nautilus-actions.h"
 #include "nautilus-application.h"
+#include "nautilus-floating-bar.h"
 #include "nautilus-location-bar.h"
 #include "nautilus-search-bar.h"
 #include "nautilus-pathbar.h"
 #include "nautilus-window-private.h"
 #include "nautilus-window-slot.h"
-#include "nautilus-navigation-window-slot.h"
 #include "nautilus-trash-bar.h"
 #include "nautilus-view-factory.h"
 #include "nautilus-x-content-bar.h"
-#include "nautilus-zoom-control.h"
-#include "nautilus-navigation-window-pane.h"
 #include <eel/eel-accessibility.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-gdk-extensions.h>
@@ -161,19 +159,16 @@ check_last_bookmark_location_matches_slot (NautilusWindowSlot *slot)
 }
 
 static void
-handle_go_back (NautilusNavigationWindowSlot *navigation_slot,
+handle_go_back (NautilusWindowSlot *slot,
 		GFile *location)
 {
-	NautilusWindowSlot *slot;
 	guint i;
 	GList *link;
 	NautilusBookmark *bookmark;
 
-	slot = NAUTILUS_WINDOW_SLOT (navigation_slot);
-
 	/* Going back. Move items from the back list to the forward list. */
-	g_assert (g_list_length (navigation_slot->back_list) > slot->location_change_distance);
-	check_bookmark_location_matches (NAUTILUS_BOOKMARK (g_list_nth_data (navigation_slot->back_list,
+	g_assert (g_list_length (slot->back_list) > slot->location_change_distance);
+	check_bookmark_location_matches (NAUTILUS_BOOKMARK (g_list_nth_data (slot->back_list,
 									     slot->location_change_distance)),
 					 location);
 	g_assert (slot->location != NULL);
@@ -183,40 +178,37 @@ handle_go_back (NautilusNavigationWindowSlot *navigation_slot,
 	check_last_bookmark_location_matches_slot (slot);
 
 	/* Use the first bookmark in the history list rather than creating a new one. */
-	navigation_slot->forward_list = g_list_prepend (navigation_slot->forward_list,
-							     slot->last_location_bookmark);
-	g_object_ref (navigation_slot->forward_list->data);
+	slot->forward_list = g_list_prepend (slot->forward_list,
+					     slot->last_location_bookmark);
+	g_object_ref (slot->forward_list->data);
 				
 	/* Move extra links from Back to Forward list */
 	for (i = 0; i < slot->location_change_distance; ++i) {
-		bookmark = NAUTILUS_BOOKMARK (navigation_slot->back_list->data);
-		navigation_slot->back_list =
-			g_list_remove (navigation_slot->back_list, bookmark);
-		navigation_slot->forward_list =
-			g_list_prepend (navigation_slot->forward_list, bookmark);
+		bookmark = NAUTILUS_BOOKMARK (slot->back_list->data);
+		slot->back_list =
+			g_list_remove (slot->back_list, bookmark);
+		slot->forward_list =
+			g_list_prepend (slot->forward_list, bookmark);
 	}
 	
 	/* One bookmark falls out of back/forward lists and becomes viewed location */
-	link = navigation_slot->back_list;
-	navigation_slot->back_list = g_list_remove_link (navigation_slot->back_list, link);
+	link = slot->back_list;
+	slot->back_list = g_list_remove_link (slot->back_list, link);
 	g_object_unref (link->data);
 	g_list_free_1 (link);
 }
 
 static void
-handle_go_forward (NautilusNavigationWindowSlot *navigation_slot,
+handle_go_forward (NautilusWindowSlot *slot,
 		   GFile *location)
 {
-	NautilusWindowSlot *slot;
 	guint i;
 	GList *link;
 	NautilusBookmark *bookmark;
 
-	slot = NAUTILUS_WINDOW_SLOT (navigation_slot);
-
 	/* Going forward. Move items from the forward list to the back list. */
-	g_assert (g_list_length (navigation_slot->forward_list) > slot->location_change_distance);
-	check_bookmark_location_matches (NAUTILUS_BOOKMARK (g_list_nth_data (navigation_slot->forward_list,
+	g_assert (g_list_length (slot->forward_list) > slot->location_change_distance);
+	check_bookmark_location_matches (NAUTILUS_BOOKMARK (g_list_nth_data (slot->forward_list,
 									     slot->location_change_distance)),
 					 location);
 	g_assert (slot->location != NULL);
@@ -225,22 +217,22 @@ handle_go_forward (NautilusNavigationWindowSlot *navigation_slot,
 	check_last_bookmark_location_matches_slot (slot);
 	
 	/* Use the first bookmark in the history list rather than creating a new one. */
-      navigation_slot->back_list = g_list_prepend (navigation_slot->back_list,
-							  slot->last_location_bookmark);
-      g_object_ref (navigation_slot->back_list->data);
+	slot->back_list = g_list_prepend (slot->back_list,
+						     slot->last_location_bookmark);
+	g_object_ref (slot->back_list->data);
 	
 	/* Move extra links from Forward to Back list */
-      for (i = 0; i < slot->location_change_distance; ++i) {
-		bookmark = NAUTILUS_BOOKMARK (navigation_slot->forward_list->data);
-		navigation_slot->forward_list =
-			g_list_remove (navigation_slot->back_list, bookmark);
-		navigation_slot->back_list =
-			g_list_prepend (navigation_slot->forward_list, bookmark);
+	for (i = 0; i < slot->location_change_distance; ++i) {
+		bookmark = NAUTILUS_BOOKMARK (slot->forward_list->data);
+		slot->forward_list =
+			g_list_remove (slot->back_list, bookmark);
+		slot->back_list =
+			g_list_prepend (slot->forward_list, bookmark);
 	}
 	
 	/* One bookmark falls out of back/forward lists and becomes viewed location */
-	link = navigation_slot->forward_list;
-	navigation_slot->forward_list = g_list_remove_link (navigation_slot->forward_list, link);
+	link = slot->forward_list;
+	slot->forward_list = g_list_remove_link (slot->forward_list, link);
 	g_object_unref (link->data);
 	g_list_free_1 (link);
 }
@@ -249,27 +241,21 @@ static void
 handle_go_elsewhere (NautilusWindowSlot *slot,
 		     GFile *location)
 {
-	NautilusNavigationWindowSlot *navigation_slot;
-
-	if (NAUTILUS_IS_NAVIGATION_WINDOW_SLOT (slot)) {
-		navigation_slot = NAUTILUS_NAVIGATION_WINDOW_SLOT (slot);
-
-		/* Clobber the entire forward list, and move displayed location to back list */
-		nautilus_navigation_window_slot_clear_forward_list (navigation_slot);
+	/* Clobber the entire forward list, and move displayed location to back list */
+	nautilus_window_slot_clear_forward_list (slot);
 		
-		if (slot->location != NULL) {
-			/* If we're returning to the same uri somehow, don't put this uri on back list. 
-			 * This also avoids a problem where set_displayed_location
-			 * didn't update last_location_bookmark since the uri didn't change.
-			 */
-			if (!g_file_equal (slot->location, location)) {
-				/* Store bookmark for current location in back list, unless there is no current location */
-				check_last_bookmark_location_matches_slot (slot);
-				/* Use the first bookmark in the history list rather than creating a new one. */
-				navigation_slot->back_list = g_list_prepend (navigation_slot->back_list,
-									     slot->last_location_bookmark);
-				g_object_ref (navigation_slot->back_list->data);
-			}
+	if (slot->location != NULL) {
+		/* If we're returning to the same uri somehow, don't put this uri on back list. 
+		 * This also avoids a problem where set_displayed_location
+		 * didn't update last_location_bookmark since the uri didn't change.
+		 */
+		if (!g_file_equal (slot->location, location)) {
+			/* Store bookmark for current location in back list, unless there is no current location */
+			check_last_bookmark_location_matches_slot (slot);
+			/* Use the first bookmark in the history list rather than creating a new one. */
+			slot->back_list = g_list_prepend (slot->back_list,
+							  slot->last_location_bookmark);
+			g_object_ref (slot->back_list->data);
 		}
 	}
 }
@@ -325,6 +311,9 @@ viewed_file_changed_callback (NautilusFile *file,
                  * file was never seen in the first place.
                  */
                 if (slot->viewed_file_seen) {
+			/* auto-show existing parent. */
+			GFile *go_to_file, *parent, *location;
+
                         /* Detecting a file is gone may happen in the
                          * middle of a pending location change, we
                          * need to cancel it before closing the window
@@ -340,37 +329,30 @@ viewed_file_changed_callback (NautilusFile *file,
                          */
 			end_location_change (slot);
 
-			if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) {
-				/* auto-show existing parent. */
-				GFile *go_to_file, *parent, *location;
-
-				go_to_file = NULL;
-				location =  nautilus_file_get_location (file);
-				parent = g_file_get_parent (location);
-				g_object_unref (location);
-				if (parent) {
-					go_to_file = nautilus_find_existing_uri_in_hierarchy (parent);
-					g_object_unref (parent);
+			go_to_file = NULL;
+			location =  nautilus_file_get_location (file);
+			parent = g_file_get_parent (location);
+			g_object_unref (location);
+			if (parent) {
+				go_to_file = nautilus_find_existing_uri_in_hierarchy (parent);
+				g_object_unref (parent);
+			}
+				
+			if (go_to_file != NULL) {
+				/* the path bar URI will be set to go_to_uri immediately
+				 * in begin_location_change, but we don't want the
+				 * inexistant children to show up anymore */
+				if (slot == slot->pane->active_slot) {
+					/* multiview-TODO also update NautilusWindowSlot
+					 * [which as of writing doesn't save/store any path bar state]
+					 */
+					nautilus_path_bar_clear_buttons (NAUTILUS_PATH_BAR (slot->pane->path_bar));
 				}
 				
-				if (go_to_file != NULL) {
-					/* the path bar URI will be set to go_to_uri immediately
-					 * in begin_location_change, but we don't want the
-					 * inexistant children to show up anymore */
-					if (slot == slot->pane->active_slot) {
-						/* multiview-TODO also update NautilusWindowSlot
-						 * [which as of writing doesn't save/store any path bar state]
-						 */
-						nautilus_path_bar_clear_buttons (NAUTILUS_PATH_BAR (NAUTILUS_NAVIGATION_WINDOW_PANE (slot->pane)->path_bar));
-					}
-
-					nautilus_window_slot_go_to (slot, go_to_file, FALSE);
-					g_object_unref (go_to_file);
-				} else {
-					nautilus_window_slot_go_home (slot, FALSE);
-				}
+				nautilus_window_slot_go_to (slot, go_to_file, FALSE);
+				g_object_unref (go_to_file);
 			} else {
-				nautilus_window_close (window);
+				nautilus_window_slot_go_home (slot, FALSE);
 			}
                 }
 	} else {
@@ -404,19 +386,16 @@ update_history (NautilusWindowSlot *slot,
         switch (type) {
         case NAUTILUS_LOCATION_CHANGE_STANDARD:
         case NAUTILUS_LOCATION_CHANGE_FALLBACK:
-                nautilus_window_slot_add_current_location_to_history_list (slot);
 		handle_go_elsewhere (slot, new_location);
                 return;
         case NAUTILUS_LOCATION_CHANGE_RELOAD:
                 /* for reload there is no work to do */
                 return;
         case NAUTILUS_LOCATION_CHANGE_BACK:
-                nautilus_window_slot_add_current_location_to_history_list (slot);
-                handle_go_back (NAUTILUS_NAVIGATION_WINDOW_SLOT (slot), new_location);
+                handle_go_back (slot, new_location);
                 return;
         case NAUTILUS_LOCATION_CHANGE_FORWARD:
-                nautilus_window_slot_add_current_location_to_history_list (slot);
-                handle_go_forward (NAUTILUS_NAVIGATION_WINDOW_SLOT (slot), new_location);
+                handle_go_forward (slot, new_location);
                 return;
         case NAUTILUS_LOCATION_CHANGE_REDIRECT:
                 /* for the redirect case, the caller can do the updating */
@@ -441,23 +420,20 @@ cancel_viewed_file_changed_callback (NautilusWindowSlot *slot)
 
 static void
 new_window_show_callback (GtkWidget *widget,
-                          gpointer user_data)
-{
-        NautilusWindow *window;
-        
-        window = NAUTILUS_WINDOW (user_data);
-        
-        nautilus_window_close (window);
+			  gpointer user_data){
+	NautilusWindow *window;
 
-        g_signal_handlers_disconnect_by_func (widget, 
-                                              G_CALLBACK (new_window_show_callback),
-                                              user_data);
+	window = NAUTILUS_WINDOW (user_data);
+	nautilus_window_close (window);
+
+	g_signal_handlers_disconnect_by_func (widget,
+					      G_CALLBACK (new_window_show_callback),
+					      user_data);
 }
 
 void
 nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 					 GFile *location,
-					 NautilusWindowOpenMode mode,
 					 NautilusWindowOpenFlags flags,
 					 GList *new_selection,
 					 NautilusWindowGoToCallback callback,
@@ -468,12 +444,11 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
         NautilusWindowPane *pane;
         NautilusWindowSlot *target_slot;
 	NautilusWindowOpenFlags slot_flags;
-        gboolean existing = FALSE;
 	GFile *old_location;
 	char *old_uri, *new_uri;
 	int new_slot_position;
 	GList *l;
-	gboolean target_navigation, target_same;
+	gboolean use_same;
 	gboolean is_desktop;
 	NautilusApplication *app;
 
@@ -481,10 +456,13 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 
         target_window = NULL;
 	target_slot = NULL;
+	use_same = FALSE;
 
+	/* this happens at startup */
 	old_uri = nautilus_window_slot_get_location_uri (slot);
 	if (old_uri == NULL) {
 		old_uri = g_strdup ("(none)");
+		use_same = TRUE;
 	}
 	new_uri = g_file_get_uri (location);
 
@@ -497,84 +475,36 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 		    (flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0));
 
 	is_desktop = NAUTILUS_IS_DESKTOP_WINDOW (window);
-	target_same = is_desktop &&
-		!nautilus_desktop_window_loaded (NAUTILUS_DESKTOP_WINDOW (window));
+
+	if (is_desktop) {
+		use_same = !nautilus_desktop_window_loaded (NAUTILUS_DESKTOP_WINDOW (window));
+	} else {
+		use_same |= g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER);
+	}
+
+	/* and if the flags specify so, this is overridden */
+	if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) != 0) {
+		use_same = FALSE;
+	}
 
 	old_location = nautilus_window_slot_get_location (slot);
 
-	switch (mode) {
-        case NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE :
-		if (g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
-			/* always use browser: if we're on the desktop the target is a new navigation window,
-			 * otherwise it's the same window.
-			 */
-			if (is_desktop) {
-				target_navigation = TRUE;
-			} else {
-				target_same = TRUE;
-			}
-		} else if (flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) {
-			/* if it's specified to open a new window, and we're not using spatial,
-			 * the target is a navigation.
-			 */
-			target_navigation = TRUE;
-		}
-                break;
-        case NAUTILUS_WINDOW_OPEN_IN_NAVIGATION :
-		target_navigation = TRUE;
-                break;
-        default :
-                g_critical ("Unknown open location mode");
-		g_object_unref (old_location);
-                return;
-        }
-
-	app = nautilus_application_dup_singleton ();
-
-	/* now get/create the window according to the mode */
-	if (target_same) {
+	/* now get/create the window */
+	if (use_same) {
 		target_window = window;
-	} else if (target_navigation) {
-		target_window = nautilus_application_create_navigation_window
+	} else {
+		app = nautilus_application_dup_singleton ();
+		target_window = nautilus_application_create_window
 			(app,
 			 NULL,
 			 gtk_window_get_screen (GTK_WINDOW (window)));
-	} else {
-		target_window = nautilus_application_get_spatial_window
-			(app,
-			 window,
-			 NULL,
-			 location,
-			 gtk_window_get_screen (GTK_WINDOW (window)),
-			 &existing);
-	}
-
-	g_object_unref (app);
-
-	/* if the spatial window is already showing, present it and set the
-	 * new selection, if present.
-	 */
-	if (existing) {
-		target_slot = target_window->details->active_pane->active_slot;
-
-		gtk_window_present (GTK_WINDOW (target_window));
-
-		if (new_selection != NULL && slot->content_view != NULL) {
-			nautilus_view_set_selection (target_slot->content_view, new_selection);
-		}
-
-		/* call the callback successfully */
-		if (callback != NULL) {
-			callback (window, NULL, user_data);
-		}
-
-		return;
+		g_object_unref (app);
 	}
 
         g_assert (target_window != NULL);
 
-	if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0 &&
-	    NAUTILUS_IS_NAVIGATION_WINDOW (window)) {
+	/* if the flags say we want a new tab, open a slot in the current window */
+	if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0) {
 		g_assert (target_window == window);
 
 		slot_flags = 0;
@@ -587,19 +517,20 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 		target_slot = nautilus_window_open_slot (window->details->active_pane, slot_flags);
 	}
 
-        if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND) != 0) {
-                if (NAUTILUS_IS_SPATIAL_WINDOW (window) && !NAUTILUS_IS_DESKTOP_WINDOW (window)) {
-                        if (gtk_widget_get_visible (GTK_WIDGET (target_window))) {
-                                nautilus_window_close (window);
-                        } else {
-                                g_signal_connect_object (target_window,
-                                                         "show",
-                                                         G_CALLBACK (new_window_show_callback),
-                                                         window,
-                                                         G_CONNECT_AFTER);
-                        }
-                }
-        }
+	/* close the current window if the flags say so */
+	if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND) != 0) {
+		if (!is_desktop) {
+			if (gtk_widget_get_visible (GTK_WIDGET (target_window))) {
+				nautilus_window_close (window);
+			} else {
+				g_signal_connect_object (target_window,
+							 "show",
+							 G_CALLBACK (new_window_show_callback),
+							 window,
+							 G_CONNECT_AFTER);
+			}
+		}
+	}
 
 	if (target_slot == NULL) {
 		if (target_window == window) {
@@ -609,8 +540,9 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 		}
 	}
 
-        if ((target_window == window && target_slot == slot &&
-	     old_location && g_file_equal (old_location, location))) {
+        if (target_window == window && target_slot == slot &&
+	    old_location && g_file_equal (old_location, location) &&
+	    !is_desktop) {
 
 		if (callback != NULL) {
 			callback (window, NULL, user_data);
@@ -769,7 +701,7 @@ begin_location_change (NautilusWindowSlot *slot,
 	end_location_change (slot);
 
 	nautilus_window_slot_set_allow_stop (slot, TRUE);
-	nautilus_window_slot_set_status (slot, " ");
+	nautilus_window_slot_set_status (slot, " ", NULL);
 
 	g_assert (slot->pending_location == NULL);
 	g_assert (slot->pending_selection == NULL);
@@ -832,96 +764,6 @@ begin_location_change (NautilusWindowSlot *slot,
 				       slot);
 }
 
-static void
-setup_new_spatial_window (NautilusWindowSlot *slot, NautilusFile *file)
-{
-	NautilusWindow *window;
-	char *show_hidden_file_setting;
-	char *geometry_string;
-	char *scroll_string;
-	gboolean maximized, sticky, above;
-	GtkAction *action;
-
-	window = slot->pane->window;
-
-	if (NAUTILUS_IS_SPATIAL_WINDOW (window) && !NAUTILUS_IS_DESKTOP_WINDOW (window)) {
-		/* load show hidden state */
-		show_hidden_file_setting = nautilus_file_get_metadata 
-			(file, NAUTILUS_METADATA_KEY_WINDOW_SHOW_HIDDEN_FILES,
-			 NULL);
-		if (show_hidden_file_setting != NULL) {
-			if (strcmp (show_hidden_file_setting, "1") == 0) {
-				window->details->show_hidden_files_mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_ENABLE;
-			} else {
-				window->details->show_hidden_files_mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DISABLE;
-			}
-
-			/* Update the UI, since we initialize it to the default */
-			action = gtk_action_group_get_action (window->details->main_action_group, NAUTILUS_ACTION_SHOW_HIDDEN_FILES);
-			gtk_action_block_activate (action);
-			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-						      window->details->show_hidden_files_mode == NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_ENABLE);
-			gtk_action_unblock_activate (action);
-		} else {
-			NAUTILUS_WINDOW (window)->details->show_hidden_files_mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT;
-		}
-		g_free (show_hidden_file_setting);
-
-		/* load the saved window geometry */
-		maximized = nautilus_file_get_boolean_metadata
-			(file, NAUTILUS_METADATA_KEY_WINDOW_MAXIMIZED, FALSE);
-		if (maximized) {
-			gtk_window_maximize (GTK_WINDOW (window));
-		} else {
-			gtk_window_unmaximize (GTK_WINDOW (window));
-		}
-
-		sticky = nautilus_file_get_boolean_metadata
-			(file, NAUTILUS_METADATA_KEY_WINDOW_STICKY, FALSE);
-		if (sticky) {
-			gtk_window_stick (GTK_WINDOW (window));
-		} else {
-			gtk_window_unstick (GTK_WINDOW (window));
-		}
-
-		above = nautilus_file_get_boolean_metadata
-			(file, NAUTILUS_METADATA_KEY_WINDOW_KEEP_ABOVE, FALSE);
-		if (above) {
-			gtk_window_set_keep_above (GTK_WINDOW (window), TRUE);
-		} else {
-			gtk_window_set_keep_above (GTK_WINDOW (window), FALSE);
-		}
-
-		geometry_string = nautilus_file_get_metadata 
-			(file, NAUTILUS_METADATA_KEY_WINDOW_GEOMETRY, NULL);
-                if (geometry_string != NULL) {
-                        eel_gtk_window_set_initial_geometry_from_string 
-                                (GTK_WINDOW (window), 
-                                 geometry_string,
-                                 NAUTILUS_SPATIAL_WINDOW_MIN_WIDTH, 
-                                 NAUTILUS_SPATIAL_WINDOW_MIN_HEIGHT,
-				 FALSE);
-                }
-                g_free (geometry_string);
-
-		if (slot->pending_selection == NULL) {
-			/* If there is no pending selection, then load the saved scroll position. */
-			scroll_string = nautilus_file_get_metadata 
-				(file, NAUTILUS_METADATA_KEY_WINDOW_SCROLL_POSITION,
-				 NULL);
-		} else {
-			/* If there is a pending selection, we want to scroll to an item in
-			 * the pending selection list. */
-			scroll_string = nautilus_file_get_uri (slot->pending_selection->data);
-		}
-
-		/* scroll_string might be NULL if there was no saved scroll position. */
-		if (scroll_string != NULL) {
-			slot->pending_scroll_to = scroll_string;
-		}
-        }
-}
-
 typedef struct {
 	GCancellable *cancellable;
 	NautilusWindowSlot *slot;
@@ -1062,10 +904,6 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 	}
 
 	if (view_id != NULL) {
-                if (!gtk_widget_get_visible (GTK_WIDGET (window)) && NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-			/* We now have the metadata to set up the window position, etc */
-			setup_new_spatial_window (slot, file);
-		}
 		create_content_view (slot, view_id);
 		g_free (view_id);
 
@@ -1122,7 +960,7 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 			/* We're missing a previous location (if opened location
 			 * in a new tab) so close it and return */
 			if (slot->location == NULL) {
-				nautilus_window_slot_close (slot);
+				nautilus_window_pane_slot_close (slot->pane, slot);
 			} else {
 				/* We disconnected this, so we need to re-connect it */
 				viewed_file = nautilus_file_get (slot->location);
@@ -1327,6 +1165,60 @@ nautilus_window_report_location_change (NautilusWindow *window)
 	}
 }
 
+static void
+real_setup_loading_floating_bar (NautilusWindowSlot *slot)
+{
+	gboolean disable_chrome;
+
+	g_object_get (slot->pane->window,
+		      "disable-chrome", &disable_chrome,
+		      NULL);
+
+	if (disable_chrome) {
+		gtk_widget_hide (slot->floating_bar);
+		return;
+	}
+
+	nautilus_floating_bar_set_label (NAUTILUS_FLOATING_BAR (slot->floating_bar),
+					 NAUTILUS_IS_SEARCH_DIRECTORY (nautilus_view_get_model (slot->content_view)) ?
+					 _("Searching...") : _("Loading..."));
+	nautilus_floating_bar_set_show_spinner (NAUTILUS_FLOATING_BAR (slot->floating_bar),
+						TRUE);
+	nautilus_floating_bar_add_action (NAUTILUS_FLOATING_BAR (slot->floating_bar),
+					  GTK_STOCK_STOP,
+					  NAUTILUS_FLOATING_BAR_ACTION_ID_STOP);
+	gtk_widget_show (slot->floating_bar);
+}
+
+static gboolean
+setup_loading_floating_bar_timeout_cb (gpointer user_data)
+{
+	NautilusWindowSlot *slot = user_data;
+
+	slot->loading_timeout_id = 0;
+	real_setup_loading_floating_bar (slot);
+
+	return FALSE;
+}
+
+static void
+setup_loading_floating_bar (NautilusWindowSlot *slot)
+{
+	/* setup loading overlay */
+	if (slot->set_status_timeout_id != 0) {
+		g_source_remove (slot->set_status_timeout_id);
+		slot->set_status_timeout_id = 0;
+	}
+
+	if (slot->loading_timeout_id != 0) {
+		g_source_remove (slot->loading_timeout_id);
+		slot->loading_timeout_id = 0;
+	}
+
+	slot->loading_timeout_id =
+		g_timeout_add (500, setup_loading_floating_bar_timeout_cb, slot);
+}
+
 /* This is called when we have decided we can actually change to the new view/location situation. */
 static void
 location_has_really_changed (NautilusWindowSlot *slot)
@@ -1338,7 +1230,7 @@ location_has_really_changed (NautilusWindowSlot *slot)
 	window = slot->pane->window;
 
 	if (slot->new_content_view != NULL) {
-		widget = nautilus_view_get_widget (slot->new_content_view);
+		widget = GTK_WIDGET (slot->new_content_view);
 		/* Switch to the new content view. */
 		if (gtk_widget_get_parent (widget) == NULL) {
 			nautilus_window_slot_set_content_view_widget (slot, slot->new_content_view);
@@ -1366,6 +1258,8 @@ location_has_really_changed (NautilusWindowSlot *slot)
 
 		g_object_unref (location_copy);
 	}
+
+	setup_loading_floating_bar (slot);
 }
 
 static void
@@ -1598,11 +1492,6 @@ update_for_new_location (NautilusWindowSlot *slot)
 		if (location_really_changed) {
 			nautilus_window_pane_sync_search_widgets (slot->pane);
 		}
-
-		if (NAUTILUS_IS_NAVIGATION_WINDOW (window) &&
-		    slot->pane == window->details->active_pane) {
-			nautilus_navigation_window_load_extension_toolbar_items (NAUTILUS_NAVIGATION_WINDOW (window));
-		}
 	}
 }
 
@@ -1634,6 +1523,18 @@ nautilus_window_report_load_complete (NautilusWindow *window,
 	}
 }
 
+static void
+remove_loading_floating_bar (NautilusWindowSlot *slot)
+{
+	if (slot->loading_timeout_id != 0) {
+		g_source_remove (slot->loading_timeout_id);
+		slot->loading_timeout_id = 0;
+	}
+
+	gtk_widget_hide (slot->floating_bar);
+	nautilus_floating_bar_cleanup_actions (NAUTILUS_FLOATING_BAR (slot->floating_bar));
+}
+
 static void
 end_location_change (NautilusWindowSlot *slot)
 {
@@ -1646,6 +1547,7 @@ end_location_change (NautilusWindowSlot *slot)
 	}
 
 	nautilus_window_slot_set_allow_stop (slot, FALSE);
+	remove_loading_floating_bar (slot);
 
         /* Now we can free pending_scroll_to, since the load_complete
          * callback already has been emitted.
@@ -1954,19 +1856,19 @@ nautilus_window_manage_views_close_slot (NautilusWindowPane *pane,
 }
 
 void
-nautilus_navigation_window_back_or_forward (NautilusNavigationWindow *window, 
-                                            gboolean back, guint distance, gboolean new_tab)
+nautilus_window_back_or_forward (NautilusWindow *window, 
+				 gboolean back,
+				 guint distance,
+				 gboolean new_tab)
 {
 	NautilusWindowSlot *slot;
-	NautilusNavigationWindowSlot *navigation_slot;
 	GList *list;
 	GFile *location;
         guint len;
         NautilusBookmark *bookmark;
 
-	slot = NAUTILUS_WINDOW (window)->details->active_pane->active_slot;
-	navigation_slot = (NautilusNavigationWindowSlot *) slot;
-	list = back ? navigation_slot->back_list : navigation_slot->forward_list;
+	slot = window->details->active_pane->active_slot;
+	list = back ? slot->back_list : slot->forward_list;
 
         len = (guint) g_list_length (list);
 
@@ -1984,7 +1886,6 @@ nautilus_navigation_window_back_or_forward (NautilusNavigationWindow *window,
 
 	if (new_tab) {
 		nautilus_window_slot_open_location_full (slot, location,
-							 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
 							 NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB,
 							 NULL, NULL, NULL);
 	} else {
